/*
 * status.c: handles the status line updating, etc for IRCII 
 *
 * Written By Michael Sandrof
 *
 * Copyright(c) 1990 
 *
 * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
 */

#include "irc.h"
#include "dcc.h"
#include "term.h"
#include "status.h"
#include "server.h"
#include "vars.h"
#include "hook.h"
#include "input.h"
#include "commands.h"
#include "window.h"
#include "screen.h"
#include "mail.h"
#include "output.h"
#include "names.h"
#include "ircaux.h"
#include "alias.h"

#ifdef Char
#undef Char
#endif
#define Char const char

	char	*update_clock 		(int);
static	Char	*convert_format 	(char *, int);
static	Char	*status_nickname 	(Window *);
static	Char	*status_query_nick 	(Window *);
static	Char	*status_right_justify 	(Window *);
static	Char	*status_chanop 		(Window *);
static	Char	*status_channel 	(Window *);
static	Char	*status_server 		(Window *);
static	Char	*status_mode 		(Window *);
static	Char	*status_umode 		(Window *);
static	Char	*status_insert_mode 	(Window *);
static	Char	*status_overwrite_mode 	(Window *);
static	Char	*status_away 		(Window *);
static	Char	*status_oper 		(Window *);
static	Char	*status_user0 		(Window *);
static	Char	*status_user1 		(Window *);
static	Char	*status_user2 		(Window *);
static	Char	*status_user3 		(Window *);
static	Char	*status_user4 		(Window *);
static	Char	*status_user5 		(Window *);
static	Char	*status_user6 		(Window *);
static	Char	*status_user7 		(Window *);
static	Char	*status_user8 		(Window *);
static	Char	*status_user9 		(Window *);
static	Char	*status_dcc 		(Window *);
static	Char	*status_hold 		(Window *);
static	Char	*status_version 	(Window *);
static	Char	*status_clock 		(Window *);
static	Char	*status_hold_lines 	(Window *);
static	Char	*status_window 		(Window *);
static	Char	*status_mail 		(Window *);
static	Char	*status_refnum 		(Window *);
static	Char	*status_null_function 	(Window *);
static	Char	*status_notify_windows 	(Window *);
static	Char	*status_voice 		(Window *);
static 	Char 	*status_cpu_saver_mode 	(Window *);
static 	Char 	*status_position 	(Window *);
static	Char	*status_scrollback	(Window *);

/*
 * Maximum number of "%" expressions in a status line format.  If you change
 * this number, you must manually change the sprintf() in make_status 
 *
 * This number will need to change to 50 when I impliment more status_user
 * variables.					-- Jake [WinterHawk] Khuon
 */
#define MAX_FUNCTIONS 40

/* The format statements to build each portion of the status line */
static	char	*mode_format 		= (char *) 0;
static	char	*umode_format 		= (char *) 0;
static	Char	*status_format[3] 	= {NULL, NULL, NULL};
static	char	*query_format 		= (char *) 0;
static	char	*clock_format 		= (char *) 0;
static	char	*hold_lines_format 	= (char *) 0;
static	char	*channel_format 	= (char *) 0;
static	char	*cpu_saver_format 	= (char *) 0;
static	char	*mail_format 		= (char *) 0;
static	char	*nick_format		= (char *) 0;
static	char	*server_format 		= (char *) 0;
static	char	*notify_format 		= (char *) 0;

/*
 * status_func: The list of status line function in the proper order for
 * display.  This list is set in convert_format() 
 */
static	Char	*(*status_func[3][MAX_FUNCTIONS]) (Window *);

/* func_cnt: the number of status line functions assigned */
static	int	func_cnt[3];


/* 
 * XXX This is bogus -- it gets set when you do /set clock_format.
 * The reason we use a global variable is to cut down on the number of
 * calls to get_string_var, which is truly bogus, but neccesary for any
 * semblance of efficiency.
 */
	char	*time_format = (char *) 0;	/* XXX Bogus XXX */
static	char	*strftime_24hour = "%R";
static	char	*strftime_12hour = "%I:%M%p";

/* update_clock: figures out the current time and returns it in a nice format */
char	*update_clock (int flag)
{
static		char	time_str[61];
static		int	min = -1;
static		int	hour = -1;

	struct	timeval	tv;
static 	struct 	tm	time_val;
static		time_t	last_minute = -1;
		time_t	hideous;

	/*
	 * This is cheating because we only call localtime() once per minute.
	 * This implicitly assumes that you only want to get the localtime
	 * info once every minute, which we do.  If you wanted to get it every
	 * second (which we dont), you DONT WANT TO DO THIS!
	 */
	get_time(&tv);
	hideous = tv.tv_sec;
#ifndef NO_CHEATING
	if (hideous / 60 > last_minute)
	{
		last_minute = hideous / 60;
		time_val = *localtime(&hideous);
	}
#else
	time_val = *localtime(&hideous);
#endif

	if (flag == RESET_TIME || time_val.tm_min != min || time_val.tm_hour != hour)
	{
		int	server = from_server;
		from_server = primary_server;

		if (time_format)	/* XXXX Bogus XXXX */
			strftime(time_str, 60, time_format, &time_val);
		else if (get_int_var(CLOCK_24HOUR_VAR))
			strftime(time_str, 60, strftime_24hour, &time_val);
		else
			strftime(time_str, 60, strftime_12hour, &time_val);

		if ((time_val.tm_min != min) || (time_val.tm_hour != hour))
		{
			hour = time_val.tm_hour;
			min = time_val.tm_min;
			do_hook(TIMER_LIST, "%02d:%02d", hour, min);
		}
		do_hook(IDLE_LIST, "%ld", (tv.tv_sec - idle_time) / 60L);
		from_server = server;
		return (time_str);
	}

	if (flag == GET_TIME)
		return(time_str);
	else
		return ((char *) 0);
}

void	reset_clock (char *unused)
{
	update_clock(RESET_TIME);
	update_all_status();
}

/*
 * convert_sub_format: This is used to convert the formats of the
 * sub-portions of the status line to a format statement specially designed
 * for that sub-portions.  convert_sub_format looks for a single occurence of
 * %c (where c is passed to the function). When found, it is replaced by "%s"
 * for use is a sprintf.  All other occurences of % followed by any other
 * character are left unchanged.  Only the first occurence of %c is
 * converted, all subsequence occurences are left unchanged.  This routine
 * mallocs the returned string. 
 */
static	char	*convert_sub_format (char *format, char c)
{
		char	buffer[BIG_BUFFER_SIZE + 1];
	static	char	bletch[] = "%% ";
		char	*ptr = (char *) 0;
		int	dont_got_it = 1;

	if (format == (char *) 0)
		return ((char *) 0);
	*buffer = (char) 0;
	while (format)
	{
		if ((ptr = (char *) index(format, '%')) != NULL)
		{
			*ptr = (char) 0;
			strmcat(buffer, format, BIG_BUFFER_SIZE);
			*(ptr++) = '%';
			if ((*ptr == c) && dont_got_it)
			{
				dont_got_it = 0;
				strmcat(buffer, "%s", BIG_BUFFER_SIZE);
			}
			else
			{
				bletch[2] = *ptr;
				strmcat(buffer, bletch, BIG_BUFFER_SIZE);
			}
			ptr++;
		}
		else
			strmcat(buffer, format, BIG_BUFFER_SIZE);
		format = ptr;
	}
	malloc_strcpy(&ptr, buffer);
	return (ptr);
}

static	Char	*convert_format (char *format, int k)
{
	char	buffer[BIG_BUFFER_SIZE + 1];
	char	*ptr;
	int	cp;

	*buffer = 0;
	while (format)
	{
		if ((ptr = (char *) index(format, '%')) != NULL)
		{
			*ptr = 0;
			strmcat(buffer, format, BIG_BUFFER_SIZE);
			*ptr = '%';
			ptr++;

			cp = func_cnt[k];
			if (cp < MAX_FUNCTIONS)
			{
/* So i alphabatized them.  Sue me. */
switch (*(ptr++))
{
	case 'A':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_away;
		break;
	case 'B':
		new_free(&hold_lines_format);
		hold_lines_format = convert_sub_format(get_string_var(STATUS_HOLD_LINES_VAR), 'B');
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_hold_lines;
		break;
	case 'C':
		new_free(&channel_format);
		channel_format = convert_sub_format(get_string_var(STATUS_CHANNEL_VAR), 'C');
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_channel;
		break;
	case 'D':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_dcc;
		break;
	case 'E':
		break; /* nothing to do */
	case 'F':
		new_free(&notify_format);
		notify_format = convert_sub_format(get_string_var(STATUS_NOTIFY_VAR), 'F');
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_notify_windows;
		break;
	case 'G': 
		break; /* nothing to do */
	case 'H':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_hold;
		break;
	case 'I':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_insert_mode;
		break;
	case 'J':
		break; /* nothing to do */
	case 'K':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_scrollback;
		break;
	case 'L':
		new_free(&cpu_saver_format);
		cpu_saver_format = convert_sub_format(get_string_var(STATUS_CPU_SAVER_VAR), 'L');
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_cpu_saver_mode;
		break;
	case 'M':
		new_free(&mail_format);
		mail_format = convert_sub_format(get_string_var(STATUS_MAIL_VAR), 'M');
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_mail;
		break;
	case 'N':
		new_free(&nick_format);
		nick_format = convert_sub_format(get_string_var(STATUS_NICK_VAR), 'N');
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_nickname;
		break;
	case 'O':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_overwrite_mode;
		break;
	case 'P':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_position;
		break;
	case 'Q':
		new_free(&query_format);
		query_format = convert_sub_format(get_string_var(STATUS_QUERY_VAR), 'Q');
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_query_nick;
		break;
	case 'R':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_refnum;
		break;
	case 'S':
		new_free(&server_format);
		server_format = convert_sub_format(get_string_var(STATUS_SERVER_VAR), 'S');
		strmcat(buffer,"%s",BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_server;
		break;
	case 'T':
		new_free(&clock_format);
		clock_format = convert_sub_format(get_string_var(STATUS_CLOCK_VAR), 'T');
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_clock;
		break;
	case 'U':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_user0;
		break;
	case 'V':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_version;
		break;
	case 'W':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_window;
		break;
	case 'X':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_user1;
		break;
	case 'Y':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_user2;
		break;
	case 'Z':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_user3;
		break;

	case '#': /* 35 */
		new_free(&umode_format);
		umode_format = convert_sub_format(get_string_var(STATUS_UMODE_VAR), '#');
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_umode;
		break;
	case '%': /* 37 */
		strmcat(buffer, "%", BIG_BUFFER_SIZE);
		break;
	case '*': /* 42 */
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_oper;
		break;
	case '+': /* 43 */
		new_free(&mode_format);
		mode_format = convert_sub_format(get_string_var(STATUS_MODE_VAR), '+');
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_mode;
		break;
	case '=': /* 61 */
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_voice;
		break;
	case '>': /* 62 */
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_right_justify;
		break;
	case '@': /* 64 */
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_chanop;
		break;

	/* status_user[0..3] are redundant of U, X, Y, and Z -- Jake [WinterHawk] Khuon */
	case '0':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_user0;
		break;
	case '1':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_user1;
		break;
	case '2':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_user2;
		break;
	case '3':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_user3;
		break;
	/* These babies are new additions. -- Jake [WinterHawk] Khuon */
	case '4':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_user4;
		break;
	case '5':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_user5;
		break;
	case '6':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_user6;
		break;
	case '7':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_user7;
		break;
	case '8':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_user8;
		break;
	case '9':
		strmcat(buffer, "%s", BIG_BUFFER_SIZE);
		status_func[k][cp++] = status_user9;
		break;

	/* no default..?? - phone, jan 1993 */
	/* empty is a good default -lynx, mar 93 */
}
			}
			else
				ptr++;
			func_cnt[k] = cp;
		}
		else
			strmcat(buffer, format, BIG_BUFFER_SIZE);
		format = ptr;
	}

	return m_strdup(buffer);
}

void	build_status (char *format)
{
	int i,k;

	for (k = 0; k < 3; k++)
	{
		new_free((char **)&status_format[k]);
		func_cnt[k] = 0;

		if (k == 0)
			format = get_string_var(STATUS_FORMAT_VAR);
		else if (k == 1)
			format = get_string_var(STATUS_FORMAT1_VAR);
		else if (k == 2)
			format = get_string_var(STATUS_FORMAT2_VAR);

		if (format)
			status_format[k] = convert_format(format, k);

		for (i = func_cnt[k]; i < MAX_FUNCTIONS; i++)
			status_func[k][i] = status_null_function;
	}
	update_all_status();
}


/*
 * This just sucked beyond words.  I was always planning on rewriting this,
 * but the crecendo of complaints with regards to this just got to be too 
 * irritating, so i fixed it early.
 */
void	make_status (Window *window)
{
	int	status_line;
	int	honor_ansi 	= get_int_var(DISPLAY_ANSI_VAR);
	u_char	buffer	    [BIG_BUFFER_SIZE + 1];
	u_char	rhs_buffer  [BIG_BUFFER_SIZE + 1];
const 	char	*func_value [MAX_FUNCTIONS];
	u_char	**output;

	for (status_line = 0; status_line < window->double_status + 1; status_line++)
	{
		char	fillchar;
		int	in_rhs = 0,
			ch_lhs = 0, ch_rhs = 0,
			pr_lhs = 0, pr_rhs = 0,
			start_rhs = -1,
			line,
			i;

		/*
		 * If status line gets to one, then that means that
		 * window->double_status is not zero.  That means that
		 * the status line we're working on is STATUS2.
		 */
		if (status_line)
			line = 2;

		/*
		 * If status_line is zero, and window->double_status is
		 * not zero (double status line is on) then we're working
		 * on STATUS1.
		 */
		else if (window->double_status)
			line = 1;

		/*
		 * So status_line is zero and window->double_status is zero.
		 * So we're working on STATUS (0).
		 */
		else
			line = 0;


		/*
		 * Sanity check:  If the status format doesnt exist, dont do
		 * anything for it.
		 */
		if (!status_format[line])
			continue;

		/*
		 * Run each of the status-generating functions from the the
		 * status list.  Note that the retval of the functions is no
		 * longer malloc()ed.  This saves 40-some odd malloc/free sets
		 * each time the status bar is updated, which is non-trivial.
		 */
		for (i = 0; i < MAX_FUNCTIONS; i++)
			func_value[i] = (status_func[line][i])(window);

		/*
		 * If the REVERSE_STATUS_LINE var is on, then put a reverse
		 * character in the first position (itll get translated to
		 * the tcap code in the output code.
		 */
		*buffer = get_int_var(REVERSE_STATUS_LINE_VAR) ? REV_TOG : ' ';

		/*
		 * Now press the status line into "buffer".  The magic about
		 * setting status_format is handled elsewhere.
		 */
		snprintf(buffer + 1, BIG_BUFFER_SIZE - 1,
		    status_format[line],
		    func_value[0], func_value[1],
		    func_value[2], func_value[3], func_value[4], func_value[5],
		    func_value[6], func_value[7], func_value[8], func_value[9],
		    func_value[10], func_value[11], func_value[12],
		    func_value[13], func_value[14], func_value[15],
		    func_value[16], func_value[17], func_value[18],
		    func_value[19], func_value[20], func_value[21],
		    func_value[22], func_value[23], func_value[24],
		    func_value[25], func_value[26], func_value[27],
		    func_value[28], func_value[29], func_value[30],
		    func_value[31], func_value[32], func_value[33],
		    func_value[34], func_value[35], func_value[36], 
		    func_value[37], func_value[38], func_value[39]); 

		if (get_int_var(STATUS_DOES_EXPANDOS_VAR))
		{
			int  af = 0;
			char *stuff;

			stuff = expand_alias(buffer, empty_string, &af, NULL);
			strmcpy(buffer, stuff, BIG_BUFFER_SIZE);
			new_free(&stuff);
		}

		/*
		 * Count out the characters.
		 * Walk the entire string, looking for nonprintable
		 * characters.  We count all the printable characters
		 * on both sides of the %> tag.
		 */
		for (i = 0; buffer[i]; i++)
		{
			/*
			 * The FIRST such %> tag is used.
			 * Using multiple %>s is bogus.
			 */
			if (buffer[i] == '\f' && start_rhs == -1)
			{
				start_rhs = i;
				in_rhs = 1;
			}

			/*
			 * Skip over color attributes if we're not
			 * doing color.
			 * XXXX Fix this so that it works right XXXXX
			 */
			else if (buffer[i] == '\003')
			{
				int c1;
				int *ch = (in_rhs) ? &ch_rhs : &ch_lhs;

				i++, (*ch)++;
				if (isdigit(buffer[i]))
				{
					c1 = buffer[i] - '0';
					i++, (*ch)++;
					if (c1 == 0 || c1 == 1)
					{
						if (isdigit(buffer[i]) &&
						   (c1 == 0 ||
						     (buffer[i] - '0' < 6)))
							i++, (*ch)++;
					}
				}
				if (buffer[i] == ',' && isdigit(buffer[i+1]))
				{
					i++, (*ch)++;
					c1 = buffer[i] - '0';
					i++, (*ch)++;
					if (c1 == 0 || c1 == 1)
					{
						if (isdigit(buffer[i]) &&
						   (c1 == 0 ||
						     (buffer[i] - '0' < 6)))
							i++, (*ch)++;
					}
				}

				/*
				 * Back over the valid char
				 */
				i--;
#if 0
				i--, (*ch)--;
#endif
			}

			/*
			 * Is it NOT a printable character?
			 */
			else if (buffer[i] == REV_TOG ||
				 buffer[i] == UND_TOG ||
				 buffer[i] == ALL_OFF ||
				 buffer[i] == BOLD_TOG ||
				 (honor_ansi && vt100_decode(buffer[i])))
			{
				if (!in_rhs)
					ch_lhs++;
				else
					ch_rhs++;
			}

			/*
			 * So it is a printable character.
			 */
			else
			{
				if (!in_rhs)
					ch_lhs++, pr_lhs++;
				else
					ch_rhs++, pr_rhs++;
			}

			/*
			 * Dont allow more than CO printable characters
			 */
			if (pr_lhs + pr_rhs >= window->screen->co)
			{
				buffer[i] = 0;
				break;
			}
		}

		/* What will we be filling with? */
		if (get_int_var(STATUS_NO_REPEAT_VAR))
			fillchar = ' ';
		else
			fillchar = buffer[ch_lhs - 1];

		/*
		 * Now if we have a rhs, then we have to adjust it.
		 */
		if (start_rhs != -1)
		{
			int numf = 0;

			strcpy(rhs_buffer, buffer + start_rhs + 1);

			numf = window->screen->co - pr_lhs - pr_rhs;
			if (numf > 0)
				strcpy(buffer + start_rhs, 
					strfill(fillchar, numf));
			else
				strcpy(buffer + start_rhs,
					empty_string);

			strcat(buffer, rhs_buffer);
		}

		/*
		 * No rhs?  If the user wants us to pad it out, do so.
		 */
		else if (get_int_var(FULL_STATUS_LINE_VAR))
		{
			int chars = window->screen->co - pr_lhs;
			int count;

			for (count = 0; count <= chars; count++)
				buffer[ch_lhs + count - 1] = fillchar;

			buffer[ch_lhs + chars] = 0;
		}

		/*
		 * This mangles the line so that it conforms to
		 * current /set's.  This saves us a whole bunch
		 * of duplicated code.  If only i could get it to
		 * count off for me...
		 */
		output = split_up_line(buffer, window->screen->co + 1);

		/*
		 * Simplified this optimization:
		 * If the status line has changed, then we 
		 * redraw it all.  If it hasnt changed, we dont redraw it.
		 * Simple?
		 */
		if (!window->status_line[status_line] ||
			strcmp(output[0], window->status_line[status_line]))
		{
			/*
			 * Roll the new back onto the old
			 */
			malloc_strcpy(&window->status_line[status_line], output[0]);

			/*
			 * Output the status line to the screen
			 */
			if (!dumb_mode)
			{
				output_screen = window->screen;
				term_move_cursor(0, window->bottom + status_line);
				output_line(output[0]);
				cursor_in_display();
				term_bold_off();
			}

			/*
			 * Or just offer it to the front end.
			 */
			else
				do_hook(STATUS_UPDATE_LIST, "%d %d %s", window->refnum, status_line, window->status_line[status_line]);
		}
	}

	cursor_to_input();
}

static	Char 	*status_nickname (Window *window)
{
static	char	my_buffer[64];

	if ((connected_to_server == 1) && !get_int_var(SHOW_STATUS_ALL_VAR)
	    && (!window->current_channel) && (window->screen->current_window != window))
		return empty_string;

	snprintf(my_buffer, 63, nick_format, get_server_nickname(window->server));
	return my_buffer;
}

static	Char	*status_server (Window *window)
{
	char	*rest,
		*name;
static	char	my_buffer[64];

	if (connected_to_server != 1)
	{
		if (window->server != -1)
		{
			if (server_format)
			{
				name = get_server_name(window->server);
				if ((rest = index(name, '.')) != NULL)
				{
					if (rest - name > 31)
						rest = name + 31;

					if (is_number(name))
						snprintf(my_buffer, 63, server_format, name);
					else
					{
						*rest = '\0';
						snprintf(my_buffer, 63, server_format, name);
						*rest = '.';
					}
				}
				else
					snprintf(my_buffer, 63, server_format, name);
			}
			else
				strcpy(my_buffer, empty_string);
		}
		else
			strcpy(my_buffer, "No Server");
	}
	else
		strcpy(my_buffer, empty_string);

	return my_buffer;
}

static	Char	*status_query_nick (Window *window)
{
	static char my_buffer[NICKNAME_LEN + 1];

	if (window->query_nick && query_format)
	{
		snprintf(my_buffer, NICKNAME_LEN, query_format, window->query_nick);
		return my_buffer;
	}

	return empty_string;
}

static	Char	*status_right_justify (Window *window)
{
	static char my_buffer[] = "\f";
	return my_buffer;
}

static	Char	*status_notify_windows (Window *window)
{
	int	doneone = 0;
	char	buf2[81];
static	char	my_buffer[81];

	if (get_int_var(SHOW_STATUS_ALL_VAR) || 
		window == window->screen->current_window)
	{
		*buf2 = 0;
		window = NULL;
		while (traverse_all_windows(&window))
		{
			if (window->miscflags & WINDOW_NOTIFIED)
			{
				if (doneone++)
					strmcat(buf2, ",", 80);
				strmcat(buf2, ltoa(window->refnum), 80);
			}
		}
	}
	if (doneone && notify_format)
	{
		snprintf(my_buffer, 80, notify_format, buf2);
		return my_buffer;
	}

	return empty_string;
}

static	Char	*status_clock (Window *window)
{
	static	char	my_buffer[81];

	if ((get_int_var(CLOCK_VAR) && clock_format)  &&
	    (get_int_var(SHOW_STATUS_ALL_VAR) ||
	    (window == window->screen->current_window)))
		snprintf(my_buffer, 80, clock_format, update_clock(GET_TIME));
	else
		strcpy(my_buffer, empty_string);

	return my_buffer;
}

static	Char	*status_mode (Window *window)
{
	char	*mode = (char *) 0;
static  char    my_buffer[81];

	*my_buffer = 0;
	if (window->current_channel)
	{
                mode = get_channel_mode((window->current_channel), window->server);
		if (mode && *mode && mode_format)
			snprintf(my_buffer, 80, mode_format, mode);
	}
	return my_buffer;
}


static	Char	*status_umode (Window *window)
{
	char	localbuf[20];
static	char	my_buffer[81];

	if ((connected_to_server == 1) && !get_int_var(SHOW_STATUS_ALL_VAR)
	    && (window->screen->current_window != window))
		strcpy(localbuf, empty_string);
	else
	{
		if (window->server >= 0)
			strmcpy(localbuf, get_umode(window->server), 19);
		else
			strcpy(localbuf, empty_string);
	}

	if (*localbuf)
		snprintf(my_buffer, 80, umode_format, localbuf);
	else
		strcpy(my_buffer, empty_string);

	return my_buffer;
}

static	Char	*status_chanop (Window *window)
{
	char	*text;

	if (window->current_channel &&
	    get_channel_oper(window->current_channel, window->server) &&
	    (text = get_string_var(STATUS_CHANOP_VAR)))
		return text;
	else
		return empty_string;
}


static	Char	*status_hold_lines (Window *window)
{
	int	num;
static	char	my_buffer[81];

	if ((num = (window->lines_held / 10) * 10))
	{
		snprintf(my_buffer, 80, hold_lines_format, ltoa(num));
		return my_buffer;
	}
	else
		return empty_string;
}

static	Char	*status_channel (Window *window)
{
	char 	channel[IRCD_BUFFER_SIZE + 1];
static	char	my_buffer[IRCD_BUFFER_SIZE + 1];

	if (window->current_channel)
	{
		int num;

		if (get_int_var(HIDE_PRIVATE_CHANNELS_VAR) && is_channel_private(window->current_channel, window->server))
			strmcpy(channel, "*private*", IRCD_BUFFER_SIZE);
		else
			strmcpy(channel, window->current_channel, IRCD_BUFFER_SIZE);

		if ((num = get_int_var(CHANNEL_NAME_WIDTH_VAR)) && (strlen(channel) > num))
			channel[num] = 0;

		snprintf(my_buffer, IRCD_BUFFER_SIZE, channel_format, channel);
		return my_buffer;
	}

	return empty_string;
}

static	Char	*status_voice (Window *window)
{
	char *text;

	if (window->current_channel &&
	    get_channel_voice(window->current_channel, window->server) &&
	    !get_channel_oper(window->current_channel, window->server) &&
	    (text = get_string_var(STATUS_VOICE_VAR)))
		return text;

	return empty_string;
}

static	Char	*status_mail (Window *window)
{
	char	*number;
static	char	my_buffer[81];

	if ((get_int_var(MAIL_VAR) && (number = check_mail()) && mail_format) &&
	    (get_int_var(SHOW_STATUS_ALL_VAR) ||
	    (window == window->screen->current_window)))
	{
		snprintf(my_buffer, 80, mail_format, number);
		return my_buffer;
	}

	return empty_string;
}

static	Char	*status_insert_mode (Window *window)
{
	char	*text;

	if (get_int_var(INSERT_MODE_VAR) && (get_int_var(SHOW_STATUS_ALL_VAR)
	    || (window->screen->current_window == window)))
		if ((text = get_string_var(STATUS_INSERT_VAR)))
			return text;

	return empty_string;
}

static	Char	*status_overwrite_mode (Window *window)
{
	char	*text;

	if (!get_int_var(INSERT_MODE_VAR) && (get_int_var(SHOW_STATUS_ALL_VAR)
	    || (window->screen->current_window == window)))
		if ((text = get_string_var(STATUS_OVERWRITE_VAR)))
			return text;

	return empty_string;
}

static	Char	*status_away (Window *window)
{
	char	*text;

	if ((connected_to_server == 1) && !get_int_var(SHOW_STATUS_ALL_VAR)
	    && (window->screen->current_window != window))
			return empty_string;
	else
	{
		if (window->server != -1 && server_list[window->server].away && (text = get_string_var(STATUS_AWAY_VAR)))
			return text;
		else
			return empty_string;
	}
}

static	Char	*status_user0 (Window *window)
{
	char	*text;

	if ((text = get_string_var(STATUS_USER_VAR)) &&
	    (get_int_var(SHOW_STATUS_ALL_VAR) ||
	    (window == window->screen->current_window)))
			return text;
	else
			return empty_string;
}

static	Char	*status_user1 (Window *window)
{
	char	*text;

        if ((text = get_string_var(STATUS_USER1_VAR)) &&
            (get_int_var(SHOW_STATUS_ALL_VAR) ||
            (window == window->screen->current_window)))
			return text;
        else
			return empty_string;
}

static	Char	*status_user2 (Window *window)
{
	char	*text;

        if ((text = get_string_var(STATUS_USER2_VAR)) &&
            (get_int_var(SHOW_STATUS_ALL_VAR) ||
            (window == window->screen->current_window)))
			return text;
        else
			return empty_string;
}

static	Char	*status_user3 (Window *window)
{
	char	*text;

        if ((text = get_string_var(STATUS_USER3_VAR)) &&
            (get_int_var(SHOW_STATUS_ALL_VAR) ||
            (window == window->screen->current_window)))
			return text;
	else
			return empty_string;
}

static	Char	*status_user4(Window *window)
{
	char	*text;

	if ((text = get_string_var(STATUS_USER4_VAR)) &&
		!(get_int_var(SHOW_STATUS_ALL_VAR)) &&
		(window != window->screen->current_window))
			return text;
	else
			return empty_string;
}

static	Char	*status_user5 (Window *window)
{
	char	*text;

	if ((text = get_string_var(STATUS_USER5_VAR)) &&
		!(get_int_var(SHOW_STATUS_ALL_VAR)) &&
		(window != window->screen->current_window))
			return text;
	else
			return empty_string;
}

static	Char	*status_user6(Window *window)
{
	char	*text;

	if ((text = get_string_var(STATUS_USER6_VAR)) &&
		!(get_int_var(SHOW_STATUS_ALL_VAR)) &&
		(window != window->screen->current_window))
			return text;
	else
			return empty_string;
}

static	Char	*status_user7(Window *window)
{
	char	*text;

	if ((text = get_string_var(STATUS_USER7_VAR)) &&
		!(get_int_var(SHOW_STATUS_ALL_VAR)) &&
		(window != window->screen->current_window))
			return text;
	else
			return empty_string;
}

static	Char	*status_user8 (Window *window)
{
	char *text;

	if ((text = get_string_var(STATUS_USER8_VAR)) &&
		!(get_int_var(SHOW_STATUS_ALL_VAR)) &&
		(window != window->screen->current_window))
			return text;
	else
			return empty_string;
}

static	Char	*status_user9 (Window *window)
{
	char	*text;

	if ((text = get_string_var(STATUS_USER9_VAR)) &&
		!(get_int_var(SHOW_STATUS_ALL_VAR)) &&
		(window != window->screen->current_window))
			return text;
	else
			return empty_string;
}

static	Char	*status_hold (Window *window)
{
	char *text;

	if (window->holding_something && (text = get_string_var(STATUS_HOLD_VAR)))
		return text;
	else
		return empty_string;
}

static	Char	*status_oper (Window *window)
{
	char *text;

	if (get_server_operator(window->server) &&
			(text = get_string_var(STATUS_OPER_VAR)) &&
			(get_int_var(SHOW_STATUS_ALL_VAR) ||
			connected_to_server != 1 || 
			(window->screen->current_window == window)))
		return text;
	else
		return empty_string;
}

static	Char	*status_window (Window *window)
{
	char *text;

	if ((number_of_windows_on_screen(window) > 1) && 
	    (window->screen->current_window == window) &&
	    (text = get_string_var(STATUS_WINDOW_VAR)))
		return text;
	else
		return empty_string;
}

static	Char	*status_refnum (Window *window)
{
	static char my_buffer[81];

	strcpy(my_buffer, window->name ? window->name : ltoa(window->refnum));
	return my_buffer;
}

static	Char	*status_version (Window *window)
{
	if ((connected_to_server == 1) && !get_int_var(SHOW_STATUS_ALL_VAR)
	    && (window->screen->current_window != window))
		return empty_string;

	return (char *)irc_version; /* XXXX */
}


static	Char	*status_null_function (Window *window)
{
	return empty_string;
}

static	Char *status_dcc (Window *window)
{
	if (window->window_level & LOG_DCC)
		return DCC_get_current_transfer();

	return empty_string;
}

static	Char *status_cpu_saver_mode (Window *window)
{
	static char my_buffer[81];

	if (cpu_saver && cpu_saver_format)
	{
		snprintf(my_buffer, 80, cpu_saver_format, "CPU");
		return my_buffer;
	}

	return empty_string;
}

static	Char *status_position (Window *window)
{
	static char my_buffer[81];

	snprintf(my_buffer, 80, "(%d-%d)", 
			window->screen->input_line,
			window->screen->input_cursor);
#if 0
					window->lines_scrolled_back,
					window->distance_from_display);
#endif
	return my_buffer;
}

static	Char *status_scrollback (Window *window)
{
	char *stuff;

	if (window->scrollback_point && 
	    (stuff = get_string_var(STATUS_SCROLLBACK_VAR)))
		return stuff;
	else
		return empty_string;
}

